<?php
/* Prevent direct access */
defined('ABSPATH') or die("You can't access this file directly.");

if (!class_exists('asp_searchController')) {
    /**
     * Class asp_searchController
     *
     * A controller slash wrapper class for the whole search process.
     *
     */
    class asp_searchController {
        /*
         * Results
         */
        private $results;

        /*
         * Array of phrases of all synonym variations
         */
        private $finalPhrases = array();

        /*
         * Constructor args
         */
        private $args;

        /*
         * Search Options
         */
        private $so;

        /*
         * Caching Options
         */
        private $co;

        function __construct( $args ) {

            $defaults = array(
                'phrase' => "",
                'id'    => 0,
                'instance' => null,
                'dontGroup' => false
            );
            $args = wp_parse_args( $args, $defaults );

            $this->args = $args;
        }

        function search() {
            $this->parseOptions();

            $sd = &$this->args['instance']['data'];
            $s = $this->args['phrase'];
            $id = $this->args['id'];

            $ra = array(
                'blogresults' => array(),

                'allbuddypresults' => array(
                    'repliesresults' => array(),
                    'groupresults' => array(),
                    'userresults' => array(),
                    'activityresults' => array()
                ),

                'alltermsresults' => array(),
                'allpageposts' => array(),
                'pageposts' => array(),
                'repliesresults' => array(),
                'allcommentsresults' => array(),
                'commentsresults' => array(),
                'userresults' => array(),
                'attachment_results' => array()
            );

            // True if only CPT search in the index table is active
            $search_only_it_posts =
                $sd['search_engine'] != 'regular' &&
                $sd['return_categories'] == 0 &&
                $sd['return_terms'] == '' &&
                $sd['return_attachments'] == 0;

            $s = $this->applyExceptions( $s );

            // Allow 0 character search on initial search phase
            if ( isset($this->so['force_order']) || isset($this->so['force_count']) )
                $sd['charcount'] = 0;

            // Allow empty search phrases only if the char count is 0
            if ( $s != "" || ($s == "" && $sd['charcount'] < 1) )
                $this->finalPhrases[] = $s;

            $this->finalPhrases = apply_filters("asp_final_phrases", $this->finalPhrases);

            // ---- Search Porcess Starts Here ----
            foreach ($this->finalPhrases as $s) {

                if (is_multisite() && $search_only_it_posts && $s != "") {
                    // Save huge amounts of server resources by not swapping all the blogs around
                    if ($_POST['asid'] == "") {
                        $_dposts      = new wpdreams_searchDemo( array() );
                        $dpageposts   = $_dposts->search( $s );
                        $ra['allpageposts'] = array_merge( $ra['allpageposts'], $dpageposts );
                    } else {
                        $this->so['switch_on_preprocess'] = 1;
                        $this->so['dont_group'] = $this->args['dontGroup'];
                        $params = array(
                            'id' => $this->args['id'],
                            'data' => $sd,
                            'options' => $this->so
                        );
                        $_posts = new asp_searchIndexTable($params);
                        $ra['pageposts'] = $_posts->search($s);

                        do_action('asp_after_pagepost_results', $s, $ra['pageposts']);
                        $ra['allpageposts'] = array_merge( $ra['allpageposts'], $ra['pageposts'] );
                    }
                } else {
                    $this->so['dont_group'] = $this->args['dontGroup'];
                    $params = array(
                        'id' => $this->args['id'],
                        'data' => $sd,
                        'options' => $this->so
                    );

                    // Regularly swap through blogs
                    foreach ($sd['selected-blogs'] as $blog) {

                        if (is_multisite()) switch_to_blog($blog);

                        aspDebug::start('search-blog-' . $blog);

                        if ($_POST['asid'] == "") {
                            $_dposts = new wpdreams_searchDemo(array());
                            $dpageposts = $_dposts->search($s);
                            $ra['allpageposts'] = array_merge($ra['allpageposts'], $dpageposts);
                        } else {

                            $_terms = new wpdreams_searchTerms($params);
                            $termsresults = $_terms->search($s);
                            $ra['alltermsresults'] = array_merge($ra['alltermsresults'], $termsresults);

                            // For exact matches the regular engine is used
                            if (
                                $sd['search_engine'] == 'regular' ||
                                isset($params['options']['set_exactonly']) ||
                                $s == ""
                            )
                                $_posts = new wpdreams_searchContent($params);
                            else
                                $_posts = new asp_searchIndexTable($params);
                            $ra['pageposts'] = $_posts->search($s);

                            $ra['allpageposts'] = array_merge($ra['allpageposts'], $ra['pageposts']);
                            do_action('asp_after_pagepost_results', $s, $ra['pageposts']);

                            $_comments = new wpdreams_searchComments($params);
                            $commentsresults = $_comments->search($s);
                            $ra['allcommentsresults'] = array_merge($ra['allcommentsresults'], $commentsresults);


                            if ( $sd['return_attachments'] ) {
                                $_attachments = new wpdreams_searchAttachments($params);
                                $ra['attachment_results'] = array_merge($ra['attachment_results'], $_attachments->search($s));
                            }

                            do_action('asp_after_attachment_results', $s, $ra['attachment_results']);

                        }

                        if (is_multisite()) restore_current_blog();
                    }

                }

                do_action('asp_after_comments_results', $s, $ra['allcommentsresults']);

                $_buddyp = new wpdreams_searchBuddyPress($params);
                $buddypresults = $_buddyp->search($s); // !!! returns array for each result (group, user, reply) !!!
                foreach ($buddypresults as $k => $v) {
                    $ra['allbuddypresults'][$k] = array_merge($ra['allbuddypresults'][$k], $v);
                }

                do_action('asp_after_buddypress_results', $s, $ra['allbuddypresults']);

                if ( $sd['user_search'] ) {
                    $_users = new wpdreams_searchUsers($params);
                    $ra['userresults'] = array_merge( $ra['userresults'], $_users->search($s) );
                }

                do_action('asp_after_user_results', $s, $ra['userresults']);


                aspDebug::stop('search-blog-' . $blog);

                $_blogs = new wpdreams_searchBlogs($params);
                $ra['blogresults'] = array_merge( $ra['blogresults'], $_blogs->search($s) );

                // Break if results count is enough
                if ((
                    count($ra['alltermsresults']) +
                    count($ra['allpageposts']) +
                    count($ra['allcommentsresults']) +
                    count($ra['allbuddypresults']['activityresults']) +
                    count($ra['allbuddypresults']['groupresults']) +
                    count($ra['blogresults']) +
                    count($ra['userresults']) +
                    count($ra['attachment_results'])
                ) >= $sd['maxresults']) break;
            }
            // ---- Search Porcess Stops Here ----


            $ra['alltermsresults'] = apply_filters('asp_terms_results', $ra['alltermsresults'], $id);
            $ra['allpageposts'] = apply_filters('asp_pagepost_results', $ra['allpageposts'], $id);
            $ra['allcommentsresults'] = apply_filters('asp_comment_results', $ra['allcommentsresults'], $id);
            $ra['allbuddypresults'] = apply_filters('asp_buddyp_results', $ra['allbuddypresults'], $id);
            $ra['blogresults'] = apply_filters('asp_blog_results', $ra['blogresults'], $id);
            $ra['userresults'] = apply_filters('asp_user_results', $ra['userresults'], $id);
            $ra['attachment_results'] = apply_filters('asp_attachment_results', $ra['attachment_results'], $id);

            /* Remove the results in polaroid mode */
            if ($sd['resultstype'] == 'polaroid' && $sd['pifnoimage'] == 'removeres') {
                foreach ($ra['allpageposts'] as $_k => $_v) {
                    if ($_v->image == null || $_v->image == '')
                        unset($ra['allpageposts'][$_k]);
                }
                foreach ($ra['allcommentsresults'] as $_k => $_v) {
                    if ($_v->image == null || $_v->image == '')
                        unset($ra['allcommentsresults'][$_k]);
                }
                foreach ($ra['allbuddypresults']['activityresults'] as $_k => $_v) {
                    if ($_v->image == null || $_v->image == '')
                        unset($ra['allbuddypresults']['activityresults'][$_k]);
                }
                foreach ($ra['allbuddypresults']['groupresults'] as $_k => $_v) {
                    if ($_v->image == null || $_v->image == '')
                        unset($ra['allbuddypresults']['groupresults'][$_k]);
                }
                foreach ($ra['blogresults'] as $_k => $_v) {
                    if ($_v->image == null || $_v->image == '')
                        unset($ra['blogresults'][$_k]);
                }
            }

            // Results as array, unordered
            $results_arr = array(
                'terms' => $ra['alltermsresults'],
                'blogs' => $ra['blogresults'],
                'bp_activities' => $ra['allbuddypresults']['activityresults'],
                'comments' => $ra['allcommentsresults'],
                'bp_groups' => $ra['allbuddypresults']['groupresults'],
                'bp_users' => $ra['userresults'],
                'post_page_cpt' => $ra['allpageposts'],
                'attachments' => $ra['attachment_results']
            );

            $results_order = $sd['results_order'];

            if (strpos($results_order, 'attachments') === false)
                $results_order .= "|attachments";

            // These keys are in the right order
            $results_order_arr = explode('|', $results_order);

            if ( $sd['resultstype'] == 'vertical' &&
                ($sd['groupby'] == 1 || $sd['groupby'] == 2) &&
                !$this->args['dontGroup']
            ) {
                $results = $this->group( $ra );
            } else {
                $results = array();
                foreach ($results_order_arr as $rk => $rv) {
                    $results = array_merge($results, $results_arr[$rv]);
                }
            }

            return $results;
        }

        function kwSuggestions() {
            $keywords = array();
            $types = array();
            $sd = &$this->args['instance']['data'];
            $results = array();

            if ($sd['searchinposts'] == 1)
                $types[] = "post";
            if ($sd['searchinpages'] == 1)
                $types[] = "page";
            if (isset($sd['selected-customtypes']) && count($sd['selected-customtypes']) > 0)
                $types = array_merge($types, $sd['selected-customtypes']);

            $remaining_count = 0;
            $possible_phrases = $this->kwPossiblePhrases( $this->args['phrase'] );

            foreach ($possible_phrases as $phrase) {
                $phrase = trim($phrase);
                if ( $phrase == "" ) continue;

                foreach (w_isset_def($sd['selected-keyword_suggestion_source'], array('google')) as $source) {
                    $remaining_count = $sd['keyword_suggestion_count'] - count($keywords);
                    if ($remaining_count <= 0) break;

                    $taxonomy = "";
                    // Check if this is a taxonomy
                    if (strpos($source, 'xtax_') !== false) {
                        $taxonomy = str_replace('xtax_', '', $source);
                        $source = "terms";
                    }

                    //$class_name = "wpd_" . $source . "KeywordSuggest";

                    $t = new  wpd_keywordSuggest($source, array(
                        'maxCount' => $remaining_count,
                        'maxCharsPerWord' => $sd['keyword_suggestion_length'],
                        'postTypes' => $types,
                        'lang' => $sd['keywordsuggestionslang'],
                        'overrideUrl' => '',
                        'taxonomy' => $taxonomy
                    ));

                    $keywords = array_merge($keywords, $t->getKeywords($phrase));
                }

                if ($remaining_count <= 0) break;
            }


            if ($keywords != false) {
                $results['keywords'] = $keywords;
                $results['nores'] = 1;
                $results = apply_filters('asp_only_keyword_results', $results);
            }

            return $results;
        }

        private function kwPossiblePhrases( $phrase ) {
            $pa = explode(" ", $phrase);
            if ( ASP_mb::strlen($pa[0]) > 3 ) {
                return array(
                    $pa[0], ASP_mb::substr($pa[0], 0, -1), ASP_mb::substr($pa[0], 1)
                );
            } else {
                return $pa;
            }
        }

        private function group( $ra ) {

            $sd = &$this->args['instance']['data'];
            $results = $ra['allpageposts'];

            $results['items'] = array();
            $results['grouped'] = 1;
            $results_order = $sd['results_order'];

            // CPM
            if (count($ra['allpageposts']) > 0) {
                $i = 9000 + strpos($results_order, 'post_page_cpt');
                $results['items'][$i] = array();
                $results['items'][$i]['data'] = $ra['allpageposts'];
                $results['items'][$i]['digdeeper'] = 1;
            }

            // Term results
            if (count($ra['alltermsresults']) > 0) {
                $num = $sd['showpostnumber'] == 1 ? " (" . count($ra['alltermsresults']) . ")" : "";
                $i = 9000 + strpos($results_order, 'terms');
                $results['items'][$i] = array();
                $results['items'][$i]['name'] = asp_icl_t( "Term group header", $sd['term_group_text'] ) . $num;
                $results['items'][$i]['data'] = $ra['alltermsresults'];
                $results['items'][$i]['digdeeper'] = 0;
                $results['grouped'] = 1;
            }

            // Blog results
            if (count($ra['blogresults']) > 0) {
                $num = $sd['showpostnumber'] == 1 ? " (" . count($ra['blogresults']) . ")" : "";
                $i = 9000 + strpos($results_order, 'blogs');
                $results['items'][$i] = array();
                $results['items'][$i]['name'] = asp_icl_t("Blog results group header", $sd['blogresultstext']) . $num;
                $results['items'][$i]['data'] = $ra['blogresults'];
                $results['items'][$i]['digdeeper'] = 0;
                $results['grouped'] = 1;
            }

            // Activity results
            if (count($ra['allbuddypresults']['activityresults']) > 0) {
                $num = $sd['showpostnumber'] == 1 ? " (" . count($ra['allbuddypresults']['activityresults']) . ")" : "";
                $i = 9000 + strpos($results_order, 'bp_activities');
                $results['items'][$i] = array();
                $results['items'][$i]['name'] = asp_icl_t( "BuddyPress activity group header",$sd['bbpressreplytext'] ) . $num;
                $results['items'][$i]['data'] = $ra['allbuddypresults']['activityresults'];
                $results['items'][$i]['digdeeper'] = 0;
                $results['grouped'] = 1;
            }

            // Comments results
            if (count($ra['allcommentsresults']) > 0) {
                $num = $sd['showpostnumber'] == 1 ? " (" . count($ra['allcommentsresults']) . ")" : "";
                $i = 9000 + strpos($results_order, 'comments');
                $results['items'][$i] = array();
                $results['items'][$i]['name'] = asp_icl_t( "Group by comments header", $sd['commentstext'] ) . $num;
                $results['items'][$i]['data'] = $ra['allcommentsresults'];
                $results['items'][$i]['digdeeper'] = 0;
                $results['grouped'] = 1;
            }

            // Buddypress groups
            if (count($ra['allbuddypresults']['groupresults']) > 0) {
                $num = $sd['showpostnumber'] == 1 ? " (" . count($ra['allbuddypresults']['groupresults']) . ")" : "";
                $i = 9000 + strpos($results_order, 'bp_groups');
                $results['items'][$i] = array();
                $results['items'][$i]['name'] = asp_icl_t("BuddyPress group header", $sd['bbpressgroupstext']) . $num;
                $results['items'][$i]['data'] = $ra['allbuddypresults']['groupresults'];
                $results['items'][$i]['digdeeper'] = 0;
                $results['grouped'] = 1;
            }

            // Users
            if (count($ra['userresults']) > 0) {
                $num = $sd['showpostnumber'] == 1 ? " (" . count($ra['userresults']) . ")" : "";
                $i = 9000 + strpos($results_order, 'bp_users');
                $results['items'][$i] = array();
                $results['items'][$i]['name'] = asp_icl_t("User group header", $sd['bbpressuserstext']) . $num;
                $results['items'][$i]['data'] = $ra['userresults'];
                $results['items'][$i]['digdeeper'] = 0;
                $results['grouped'] = 1;
            }

            // Attachments
            if (count($ra['attachment_results']) > 0) {
                $num = $sd['showpostnumber'] == 1 ? " (" . count($ra['attachment_results']) . ")" : "";
                $i = 9000 + strpos($results_order, 'attachments');
                $results['items'][$i] = array();
                $results['items'][$i]['name'] = asp_icl_t( "Attachment group header", $sd['attachment_group_text'] ) . $num;
                $results['items'][$i]['data'] = $ra['attachment_results'];
                $results['items'][$i]['digdeeper'] = 0;
                $results['grouped'] = 1;
            }

            if (isset($results['items']) && count($results['items'])>0)
                ksort($results['items']);
            else
                $results = array();

            return $results;
        }

        private function parseOptions() {

            $sd = &$this->args['instance']['data'];

            $this->co = get_option('asp_caching', wd_asp()->options['asp_caching_def']);

            // See if we post the preview data through
            if (!empty($_POST['asp_preview_options']) && current_user_can("manage_options")) {
                //$sd = array_merge($sd, unserialize(base64_decode($_POST['asp_preview_options'])));
                $sd = array_merge($sd, unserialize(base64_decode($_POST['asp_preview_options'])));
            }

            $sd['image_options'] = array(
                'image_cropping' => w_isset_def($this->co['image_cropping'], 1),
                'show_images' => $sd['show_images'],
                'image_bg_color' => $sd['image_bg_color'],
                'image_transparency' => $sd['image_transparency'],
                'apply_content_filter' => $sd['image_apply_content_filter'],
                'image_width' => $sd['image_width'],
                'image_height' => $sd['image_height'],
                'image_source1' => $sd['image_source1'],
                'image_source2' => $sd['image_source2'],
                'image_source3' => $sd['image_source3'],
                'image_source4' => $sd['image_source4'],
                'image_source5' => $sd['image_source5'],
                'image_default' => $sd['image_default'],
                'image_custom_field' => $sd['image_custom_field']
            );

            if (isset($_POST['asp_get_as_array']))
                $sd['image_options']['show_images'] = 0;

            // ----------------- Recalculate image width/height ---------------
            switch ($sd['resultstype']) {
                case "horizontal":
                    /* Same width as height */
                    $sd['image_options']['image_width'] = wpdreams_width_from_px($sd['hreswidth']);
                    $sd['image_options']['image_height'] = wpdreams_width_from_px($sd['hreswidth']);
                    break;
                case "polaroid":
                    $sd['image_options']['image_width'] = intval($sd['preswidth']);
                    $sd['image_options']['image_height'] = intval($sd['preswidth']);
                    break;
                case "isotopic":
                    $sd['image_options']['image_width'] = intval($sd['i_item_width'] * 1.5);
                    $sd['image_options']['image_height'] = intval($sd['i_item_height'] * 1.5);
                    break;
            }

            if (isset($sd['selected-imagesettings'])) {
                $sd['settings-imagesettings'] = $sd['selected-imagesettings'];
            }
            /*if (isset($search) && $sd['exactonly']!=1) {
              $_s = explode(" ", $s);
            }*/
            if (isset($_POST['options'])) {
                if (is_array($_POST['options']))
                    $this->so = $_POST['options'];
                else
                    parse_str($_POST['options'], $this->so);
            }

            $sd['selected-blogs'] = w_isset_def($sd['selected-blogs'], array(0 => get_current_blog_id()));

            if ($sd['selected-blogs'] === "all") {
                if (is_multisite())
                    $sd['selected-blogs'] = wpdreams_get_blog_list(0, "all", true);
                else
                    $sd['selected-blogs'] = array(0 => get_current_blog_id());
            }
            if (count($sd['selected-blogs']) <= 0) {
                $sd['selected-blogs'] = array(0 => get_current_blog_id());
            }

            do_action('asp_before_search', $this->args['phrase']);

            if (w_isset_def($this->co['caching'], 0) == 1 && ASP_DEBUG != 1) {
                $this->getResCache();
            }

        }

        private function applyExceptions( $s ) {
            $sd = &$this->args['instance']['data'];

            if ($sd["kw_exceptions"] == "" && $sd["kw_exceptions_e"] == "") return $s;

            if ($sd["kw_exceptions"] != "") {
                $exceptions = str_replace(array(" ,", ", ", " , "), ",", $sd["kw_exceptions"]);
                $s = trim( str_replace( explode(",", $exceptions), "", $s) );
                $s = preg_replace('/\s+/', ' ', $s);
            }

            if ($sd["kw_exceptions_e"] != "") {
                $exceptions = str_replace(array(" ,", ", ", " , "), ",", $sd["kw_exceptions_e"]);
                $exceptions = str_replace(",", "|", $exceptions);
                $s = trim( preg_replace('/\b' . $exceptions . '\b/u', "", $s) );
                $s = preg_replace('/\s+/', ' ', $s);
            }

            return $s;
        }

        private function getResCache() {
            $filename = ASP_PATH . DIRECTORY_SEPARATOR . "cache" . DIRECTORY_SEPARATOR . md5(json_encode($this->so) . $this->args['phrase'] . $_POST['asp_inst_id']) . ".wp";
            $this->textcache = new wpdreamsTextCache($filename, w_isset_def($this->co['cachinginterval'], 3600) * 60);
            $cache_content = $this->textcache->getCache();
            if ($cache_content != false) {
                $cache_content = apply_filters('asp_cached_content', $cache_content);
                do_action('asp_after_search', $this->args['phrase'], json_decode($cache_content, true));
                print "cached(" . date("F d Y H:i:s.", filemtime($filename)) . ")";
                print_r($cache_content);
                die;
            }
        }

        function setResCache( $html_results ) {
            if ( isset($this->textcache) )
                return $this->textcache->setCache('!!ASPSTART!!' . $html_results . "!!ASPEND!!");
        }
    }
}